@@ -1,27 +1,38 @@
 """codspeed benchmarks for client requests."""
 
 import asyncio
-from http.cookies import Morsel
+from http.cookies import BaseCookie
 from typing import Union
 
+from multidict import CIMultiDict
 from pytest_codspeed import BenchmarkFixture
 from yarl import URL
 
-from aiohttp.client_reqrep import ClientRequest
+from aiohttp.client_reqrep import ClientRequest, ClientResponse
+from aiohttp.cookiejar import CookieJar
+from aiohttp.helpers import TimerNoop
 from aiohttp.http_writer import HttpVersion11
+from aiohttp.tracing import Trace
 
 
 def test_client_request_update_cookies(
     loop: asyncio.AbstractEventLoop, benchmark: BenchmarkFixture
 ) -> None:
-    req = ClientRequest("get", URL("http://python.org"), loop=loop)
-    morsel: "Morsel[str]" = Morsel()
-    morsel.set(key="string", val="Another string", coded_val="really")
-    morsel_cookie = {"str": morsel}
+    url = URL("http://python.org")
+
+    async def setup():
+        cookie_jar = CookieJar()
+        cookie_jar.update_cookies({"string": "Another string"})
+        cookies = cookie_jar.filter_cookies(url)
+        assert cookies["string"].value == "Another string"
+        req = ClientRequest("get", url, loop=loop)
+        return req, cookies
+
+    req, cookies = loop.run_until_complete(setup())
 
     @benchmark
     def _run() -> None:
-        req.update_cookies(cookies=morsel_cookie)
+        req.update_cookies(cookies=cookies)
 
 
 def test_create_client_request_with_cookies(
@@ -29,15 +40,39 @@ def test_create_client_request_with_cookies(
 ) -> None:
     url = URL("http://python.org")
 
+    async def setup():
+        cookie_jar = CookieJar()
+        cookie_jar.update_cookies({"cookie": "value"})
+        cookies = cookie_jar.filter_cookies(url)
+        assert cookies["cookie"].value == "value"
+        return cookies
+
+    cookies = loop.run_until_complete(setup())
+    timer = TimerNoop()
+    traces: list[Trace] = []
+    headers = CIMultiDict[str]()
+
     @benchmark
     def _run() -> None:
         ClientRequest(
             method="get",
             url=url,
             loop=loop,
-            headers=None,
+            params=None,
+            skip_auto_headers=None,
+            response_class=ClientResponse,
+            proxy=None,
+            proxy_auth=None,
+            proxy_headers=None,
+            timer=timer,
+            session=None,
+            ssl=True,
+            traces=traces,
+            trust_env=False,
+            server_hostname=None,
+            headers=headers,
             data=None,
-            cookies={"cookie": "value"},
+            cookies=cookies,
             auth=None,
             version=HttpVersion11,
             compress=False,
@@ -50,6 +85,10 @@ def test_create_client_request_with_headers(
     loop: asyncio.AbstractEventLoop, benchmark: BenchmarkFixture
 ) -> None:
     url = URL("http://python.org")
+    timer = TimerNoop()
+    traces: list[Trace] = []
+    headers = CIMultiDict({"header": "value", "another": "header"})
+    cookies = BaseCookie[str]()
 
     @benchmark
     def _run() -> None:
@@ -57,9 +96,21 @@ def test_create_client_request_with_headers(
             method="get",
             url=url,
             loop=loop,
-            headers={"header": "value", "another": "header"},
+            params=None,
+            skip_auto_headers=None,
+            response_class=ClientResponse,
+            proxy=None,
+            proxy_auth=None,
+            proxy_headers=None,
+            timer=timer,
+            session=None,
+            ssl=True,
+            traces=traces,
+            trust_env=False,
+            server_hostname=None,
+            headers=headers,
             data=None,
-            cookies=None,
+            cookies=cookies,
             auth=None,
             version=HttpVersion11,
             compress=False,
