@@ -6,50 +6,50 @@ from aiohttp import web
 from aiohttp.web_urldispatcher import View
 
 
-def test_ctor():
+def test_ctor() -> None:
     request = mock.Mock()
     view = View(request)
     assert view.request is request
 
 
-async def test_render_ok():
-    resp = web.Response(text='OK')
+async def test_render_ok() -> None:
+    resp = web.Response(text="OK")
 
     class MyView(View):
         async def get(self):
             return resp
 
     request = mock.Mock()
-    request.method = 'GET'
+    request.method = "GET"
     resp2 = await MyView(request)
     assert resp is resp2
 
 
-async def test_render_unknown_method():
-
+async def test_render_unknown_method() -> None:
     class MyView(View):
         async def get(self):
-            return web.Response(text='OK')
+            return web.Response(text="OK")
+
         options = get
 
     request = mock.Mock()
-    request.method = 'UNKNOWN'
+    request.method = "UNKNOWN"
     with pytest.raises(web.HTTPMethodNotAllowed) as ctx:
         await MyView(request)
-    assert ctx.value.headers['allow'] == 'GET,OPTIONS'
+    assert ctx.value.headers["allow"] == "GET,OPTIONS"
     assert ctx.value.status == 405
 
 
-async def test_render_unsupported_method():
-
+async def test_render_unsupported_method() -> None:
     class MyView(View):
         async def get(self):
-            return web.Response(text='OK')
+            return web.Response(text="OK")
+
         options = delete = get
 
     request = mock.Mock()
-    request.method = 'POST'
+    request.method = "POST"
     with pytest.raises(web.HTTPMethodNotAllowed) as ctx:
         await MyView(request)
-    assert ctx.value.headers['allow'] == 'DELETE,GET,OPTIONS'
+    assert ctx.value.headers["allow"] == "DELETE,GET,OPTIONS"
     assert ctx.value.status == 405
