@@ -5,13 +5,15 @@ from unittest import mock
 
 import aiosignal
 import pytest
+from yarl import URL
 
 from aiohttp import helpers, web
+from aiohttp.pytest_plugin import AiohttpClient
 from aiohttp.test_utils import make_mocked_request
 
 
 @pytest.fixture
-def buf():
+def buf() -> bytearray:
     return bytearray()
 
 
@@ -188,11 +190,38 @@ def test_empty_body_304() -> None:
     resp.body is None
 
 
-def test_link_header_451(buf) -> None:
-    resp = web.HTTPUnavailableForLegalReasons(link="http://warning.or.kr/")
+def test_no_link_451() -> None:
+    with pytest.raises(TypeError):
+        web.HTTPUnavailableForLegalReasons()  # type: ignore[call-arg]
 
-    assert "http://warning.or.kr/" == resp.link
-    assert '<http://warning.or.kr/>; rel="blocked-by"' == resp.headers["Link"]
+
+def test_link_none_451() -> None:
+    resp = web.HTTPUnavailableForLegalReasons(link=None)
+    assert resp.link is None
+    assert "Link" not in resp.headers
+
+
+def test_link_empty_451() -> None:
+    resp = web.HTTPUnavailableForLegalReasons(link="")
+    assert resp.link is None
+    assert "Link" not in resp.headers
+
+
+def test_link_str_451() -> None:
+    resp = web.HTTPUnavailableForLegalReasons(link="http://warning.or.kr/")
+    assert resp.link == URL("http://warning.or.kr/")
+    assert resp.headers["Link"] == '<http://warning.or.kr/>; rel="blocked-by"'
+
+
+def test_link_url_451() -> None:
+    resp = web.HTTPUnavailableForLegalReasons(link=URL("http://warning.or.kr/"))
+    assert resp.link == URL("http://warning.or.kr/")
+    assert resp.headers["Link"] == '<http://warning.or.kr/>; rel="blocked-by"'
+
+
+def test_link_CRLF_451() -> None:
+    resp = web.HTTPUnavailableForLegalReasons(link="http://warning.or.kr/\r\n")
+    assert "\r\n" not in resp.headers["Link"]
 
 
 def test_HTTPException_retains_cause() -> None:
@@ -206,7 +235,7 @@ def test_HTTPException_retains_cause() -> None:
     assert "direct cause" in tb
 
 
-async def test_HTTPException_retains_cookie(aiohttp_client):
+async def test_HTTPException_retains_cookie(aiohttp_client: AiohttpClient) -> None:
     @web.middleware
     async def middleware(request, handler):
         try:
@@ -241,3 +270,8 @@ def test_unicode_text_body_unauthorized() -> None:
     ):
         resp = web.HTTPUnauthorized(body="text")
     assert resp.status == 401
+
+
+def test_multiline_reason() -> None:
+    with pytest.raises(ValueError, match=r"Reason cannot contain \\n"):
+        web.HTTPOk(reason="Bad\r\nInjected-header: foo")
