@@ -4,8 +4,7 @@ import json
 import pathlib
 import socket
 import sys
-import zlib
-from typing import Any, NoReturn, Optional
+from typing import Any, Dict, Generator, NoReturn, Optional, Tuple
 from unittest import mock
 
 import pytest
@@ -22,9 +21,9 @@ from aiohttp import (
     multipart,
     web,
 )
+from aiohttp.compression_utils import ZLibBackend, ZLibCompressObjProtocol
 from aiohttp.hdrs import CONTENT_LENGTH, CONTENT_TYPE, TRANSFER_ENCODING
-from aiohttp.pytest_plugin import AiohttpClient
-from aiohttp.test_utils import make_mocked_coro
+from aiohttp.pytest_plugin import AiohttpClient, AiohttpServer
 from aiohttp.typedefs import Handler
 from aiohttp.web_protocol import RequestHandler
 
@@ -403,7 +402,7 @@ async def test_post_single_file(aiohttp_client) -> None:
         data = await request.post()
         assert ["data.unknown_mime_type"] == list(data.keys())
         for fs in data.values():
-            check_file(fs)
+            await asyncio.to_thread(check_file, fs)
             fs.file.close()
         resp = web.Response(body=b"OK")
         return resp
@@ -429,9 +428,9 @@ async def test_files_upload_with_same_key(aiohttp_client) -> None:
         for _file in files:
             assert not _file.file.closed
             if _file.filename == "test1.jpeg":
-                assert _file.file.read() == b"binary data 1"
+                assert await asyncio.to_thread(_file.file.read) == b"binary data 1"
             if _file.filename == "test2.jpeg":
-                assert _file.file.read() == b"binary data 2"
+                assert await asyncio.to_thread(_file.file.read) == b"binary data 2"
             file_names.add(_file.filename)
             _file.file.close()
         assert len(files) == 2
@@ -471,7 +470,7 @@ async def test_post_files(aiohttp_client) -> None:
         data = await request.post()
         assert ["data.unknown_mime_type", "conftest.py"] == list(data.keys())
         for fs in data.values():
-            check_file(fs)
+            await asyncio.to_thread(check_file, fs)
             fs.file.close()
         resp = web.Response(body=b"OK")
         return resp
@@ -696,9 +695,8 @@ async def test_http11_keep_alive_default(aiohttp_client) -> None:
     await resp.release()
 
 
-@pytest.mark.xfail
-async def test_http10_keep_alive_default(aiohttp_client) -> None:
-    async def handler(request):
+async def test_http10_keep_alive_default(aiohttp_client: AiohttpClient) -> None:
+    async def handler(request: web.Request) -> web.Response:
         return web.Response()
 
     app = web.Application()
@@ -758,7 +756,7 @@ async def test_upload_file(aiohttp_client) -> None:
 
     async def handler(request):
         form = await request.post()
-        raw_data = form["file"].file.read()
+        raw_data = await asyncio.to_thread(form["file"].file.read)
         form["file"].file.close()
         assert data == raw_data
         return web.Response()
@@ -781,7 +779,7 @@ async def test_upload_file_object(aiohttp_client) -> None:
 
     async def handler(request):
         form = await request.post()
-        raw_data = form["file"].file.read()
+        raw_data = await asyncio.to_thread(form["file"].file.read)
         form["file"].file.close()
         assert data == raw_data
         return web.Response()
@@ -907,10 +905,10 @@ async def test_response_with_async_gen(aiohttp_client, fname) -> None:
 
     async def stream(f_name):
         with f_name.open("rb") as f:
-            data = f.read(100)
+            data = await asyncio.to_thread(f.read, 100)
             while data:
                 yield data
-                data = f.read(100)
+                data = await asyncio.to_thread(f.read, 100)
 
     async def handler(request):
         headers = {"Content-Length": str(data_size)}
@@ -941,10 +939,10 @@ async def test_response_with_streamer(aiohttp_client, fname) -> None:
         @aiohttp.streamer
         async def stream(writer, f_name):
             with f_name.open("rb") as f:
-                data = f.read(100)
+                data = await asyncio.to_thread(f.read, 100)
                 while data:
                     await writer.write(data)
-                    data = f.read(100)
+                    data = await asyncio.to_thread(f.read, 100)
 
     async def handler(request):
         headers = {"Content-Length": str(data_size)}
@@ -970,10 +968,10 @@ async def test_response_with_async_gen_no_params(aiohttp_client, fname) -> None:
 
     async def stream():
         with fname.open("rb") as f:
-            data = f.read(100)
+            data = await asyncio.to_thread(f.read, 100)
             while data:
                 yield data
-                data = f.read(100)
+                data = await asyncio.to_thread(f.read, 100)
 
     async def handler(request):
         headers = {"Content-Length": str(data_size)}
@@ -1004,10 +1002,10 @@ async def test_response_with_streamer_no_params(aiohttp_client, fname) -> None:
         @aiohttp.streamer
         async def stream(writer):
             with fname.open("rb") as f:
-                data = f.read(100)
+                data = await asyncio.to_thread(f.read, 100)
                 while data:
                     await writer.write(data)
-                    data = f.read(100)
+                    data = await asyncio.to_thread(f.read, 100)
 
     async def handler(request):
         headers = {"Content-Length": str(data_size)}
@@ -1135,19 +1133,30 @@ async def test_response_with_payload_stringio(aiohttp_client, fname) -> None:
     await resp.release()
 
 
-@pytest.mark.parametrize(
-    "compressor,encoding",
-    [
-        (zlib.compressobj(wbits=16 + zlib.MAX_WBITS), "gzip"),
-        (zlib.compressobj(wbits=zlib.MAX_WBITS), "deflate"),
-        # Actually, wrong compression format, but
-        # should be supported for some legacy cases.
-        (zlib.compressobj(wbits=-zlib.MAX_WBITS), "deflate"),
-    ],
-)
+@pytest.fixture(params=["gzip", "deflate", "deflate-raw"])
+def compressor_case(
+    request: pytest.FixtureRequest,
+    parametrize_zlib_backend: None,
+) -> Generator[Tuple[ZLibCompressObjProtocol, str], None, None]:
+    encoding: str = request.param
+    max_wbits: int = ZLibBackend.MAX_WBITS
+
+    encoding_to_wbits: Dict[str, int] = {
+        "deflate": max_wbits,
+        "deflate-raw": -max_wbits,
+        "gzip": 16 + max_wbits,
+    }
+
+    compressor = ZLibBackend.compressobj(wbits=encoding_to_wbits[encoding])
+    yield (compressor, "deflate" if encoding.startswith("deflate") else encoding)
+
+
 async def test_response_with_precompressed_body(
-    aiohttp_client, compressor, encoding
+    aiohttp_client: AiohttpClient,
+    compressor_case: Tuple[ZLibCompressObjProtocol, str],
 ) -> None:
+    compressor, encoding = compressor_case
+
     async def handler(request):
         headers = {"Content-Encoding": encoding}
         data = compressor.compress(b"mydata") + compressor.flush()
@@ -1688,9 +1697,7 @@ async def test_app_max_client_size(aiohttp_client) -> None:
         resp = await client.post("/", data=data)
     assert 413 == resp.status
     resp_text = await resp.text()
-    assert (
-        "Maximum request body size 1048576 exceeded, " "actual body size" in resp_text
-    )
+    assert "Maximum request body size 1048576 exceeded, actual body size" in resp_text
     # Maximum request body size X exceeded, actual body size X
     body_size = int(resp_text.split()[-1])
     assert body_size >= max_size
@@ -1722,9 +1729,7 @@ async def test_app_max_client_size_adjusted(aiohttp_client) -> None:
         resp = await client.post("/", data=too_large_data)
     assert 413 == resp.status
     resp_text = await resp.text()
-    assert (
-        "Maximum request body size 2097152 exceeded, " "actual body size" in resp_text
-    )
+    assert "Maximum request body size 2097152 exceeded, actual body size" in resp_text
     # Maximum request body size X exceeded, actual body size X
     body_size = int(resp_text.split()[-1])
     assert body_size >= custom_max_size
@@ -1951,6 +1956,10 @@ async def test_response_context_manager_error(aiohttp_server) -> None:
             await resp.read()
     assert resp.closed
 
+    # Wait for any pending operations to complete
+    await resp.wait_for_close()
+
+    assert session._connector is not None
     assert len(session._connector._conns) == 1
 
     await session.close()
@@ -2019,15 +2028,14 @@ async def test_iter_any(aiohttp_server) -> None:
             assert resp.status == 200
 
 
-async def test_request_tracing(aiohttp_server) -> None:
-
-    on_request_start = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
-    on_request_end = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
-    on_dns_resolvehost_start = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
-    on_dns_resolvehost_end = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
-    on_request_redirect = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
-    on_connection_create_start = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
-    on_connection_create_end = mock.Mock(side_effect=make_mocked_coro(mock.Mock()))
+async def test_request_tracing(aiohttp_server: AiohttpServer) -> None:
+    on_request_start = mock.AsyncMock()
+    on_request_end = mock.AsyncMock()
+    on_dns_resolvehost_start = mock.AsyncMock()
+    on_dns_resolvehost_end = mock.AsyncMock()
+    on_request_redirect = mock.AsyncMock()
+    on_connection_create_start = mock.AsyncMock()
+    on_connection_create_end = mock.AsyncMock()
 
     async def redirector(request):
         raise web.HTTPFound(location=URL("/redirected"))
@@ -2092,6 +2100,10 @@ async def test_request_tracing(aiohttp_server) -> None:
     await client.close()
 
 
+@pytest.mark.skipif(
+    hasattr(sys, "_is_gil_enabled") and not sys._is_gil_enabled(),
+    reason="Fails to capture the warning",
+)
 async def test_return_http_exception_deprecated(aiohttp_client) -> None:
     async def handler(request):
         return web.HTTPForbidden()
@@ -2194,6 +2206,7 @@ async def test_read_bufsize(aiohttp_client) -> None:
 @pytest.mark.parametrize(
     "auto_decompress,len_of", [(True, "uncompressed"), (False, "compressed")]
 )
+@pytest.mark.usefixtures("parametrize_zlib_backend")
 async def test_auto_decompress(
     aiohttp_client,
     auto_decompress,
@@ -2208,7 +2221,7 @@ async def test_auto_decompress(
 
     client = await aiohttp_client(app)
     uncompressed = b"dataaaaaaaaaaaaaaaaaaaaaaaaa"
-    compressor = zlib.compressobj(wbits=16 + zlib.MAX_WBITS)
+    compressor = ZLibBackend.compressobj(wbits=16 + ZLibBackend.MAX_WBITS)
     compressed = compressor.compress(uncompressed) + compressor.flush()
     assert len(compressed) != len(uncompressed)
     headers = {"content-encoding": "gzip"}
@@ -2329,3 +2342,41 @@ async def test_keepalive_race_condition(aiohttp_client: Any) -> None:
         # Make 2nd request which will hit the race condition.
         async with client.get("/") as resp:
             assert resp.status == 200
+
+
+async def test_keepalive_expires_on_time(aiohttp_client: AiohttpClient) -> None:
+    """Test that the keepalive handle expires on time."""
+
+    async def handler(request: web.Request) -> web.Response:
+        body = await request.read()
+        assert b"" == body
+        return web.Response(body=b"OK")
+
+    app = web.Application()
+    app.router.add_route("GET", "/", handler)
+
+    connector = aiohttp.TCPConnector(limit=1)
+    client = await aiohttp_client(app, connector=connector)
+
+    loop = asyncio.get_running_loop()
+    now = loop.time()
+
+    # Patch loop time so we can control when the keepalive timeout is processed
+    with mock.patch.object(loop, "time") as loop_time_mock:
+        loop_time_mock.return_value = now
+        resp1 = await client.get("/")
+        await resp1.read()
+        request_handler = client.server.handler.connections[0]
+
+        # Ensure the keep alive handle is set
+        assert request_handler._keepalive_handle is not None
+
+        # Set the loop time to exactly the keepalive timeout
+        loop_time_mock.return_value = request_handler._next_keepalive_close_time
+
+        # sleep twice to ensure the keep alive timeout is processed
+        await asyncio.sleep(0)
+        await asyncio.sleep(0)
+
+        # Ensure the keep alive handle expires
+        assert request_handler._keepalive_handle is None
