@@ -5,15 +5,16 @@ from unittest import mock
 
 import pytest
 
-from aiohttp import DataQueue, WSMessage
+from aiohttp import WSMsgType
+from aiohttp._websocket.reader import WebSocketDataQueue
+from aiohttp.compression_utils import ZLibBackend
 from aiohttp.http import WebSocketReader, WebSocketWriter
-from aiohttp.test_utils import make_mocked_coro
 
 
 @pytest.fixture
 def protocol():
     ret = mock.Mock()
-    ret._drain_helper = make_mocked_coro()
+    ret._drain_helper = mock.AsyncMock()
     return ret
 
 
@@ -29,33 +30,33 @@ def writer(protocol, transport):
     return WebSocketWriter(protocol, transport, use_mask=False)
 
 
-async def test_pong(writer) -> None:
-    await writer.pong()
-    writer.transport.write.assert_called_with(b"\x8a\x00")
+async def test_pong(writer: WebSocketWriter) -> None:
+    await writer.send_frame(b"", WSMsgType.PONG)
+    writer.transport.write.assert_called_with(b"\x8a\x00")  # type: ignore[attr-defined]
 
 
-async def test_ping(writer) -> None:
-    await writer.ping()
-    writer.transport.write.assert_called_with(b"\x89\x00")
+async def test_ping(writer: WebSocketWriter) -> None:
+    await writer.send_frame(b"", WSMsgType.PING)
+    writer.transport.write.assert_called_with(b"\x89\x00")  # type: ignore[attr-defined]
 
 
-async def test_send_text(writer) -> None:
-    await writer.send(b"text")
-    writer.transport.write.assert_called_with(b"\x81\x04text")
+async def test_send_text(writer: WebSocketWriter) -> None:
+    await writer.send_frame(b"text", WSMsgType.TEXT)
+    writer.transport.write.assert_called_with(b"\x81\x04text")  # type: ignore[attr-defined]
 
 
-async def test_send_binary(writer) -> None:
-    await writer.send("binary", True)
-    writer.transport.write.assert_called_with(b"\x82\x06binary")
+async def test_send_binary(writer: WebSocketWriter) -> None:
+    await writer.send_frame(b"binary", WSMsgType.BINARY)
+    writer.transport.write.assert_called_with(b"\x82\x06binary")  # type: ignore[attr-defined]
 
 
-async def test_send_binary_long(writer) -> None:
-    await writer.send(b"b" * 127, True)
-    assert writer.transport.write.call_args[0][0].startswith(b"\x82~\x00\x7fb")
+async def test_send_binary_long(writer: WebSocketWriter) -> None:
+    await writer.send_frame(b"b" * 127, WSMsgType.BINARY)
+    assert writer.transport.write.call_args[0][0].startswith(b"\x82~\x00\x7fb")  # type: ignore[attr-defined]
 
 
-async def test_send_binary_very_long(writer) -> None:
-    await writer.send(b"b" * 65537, True)
+async def test_send_binary_very_long(writer: WebSocketWriter) -> None:
+    await writer.send_frame(b"b" * 65537, WSMsgType.BINARY)
     assert (
         writer.transport.write.call_args_list[0][0][0]
         == b"\x82\x7f\x00\x00\x00\x00\x00\x01\x00\x01"
@@ -79,34 +80,58 @@ async def test_send_text_masked(protocol, transport) -> None:
     writer = WebSocketWriter(
         protocol, transport, use_mask=True, random=random.Random(123)
     )
-    await writer.send(b"text")
-    writer.transport.write.assert_called_with(b"\x81\x84\rg\xb3fy\x02\xcb\x12")
+    await writer.send_frame(b"text", WSMsgType.TEXT)
+    writer.transport.write.assert_called_with(b"\x81\x84\rg\xb3fy\x02\xcb\x12")  # type: ignore[attr-defined]
 
 
+@pytest.mark.usefixtures("parametrize_zlib_backend")
 async def test_send_compress_text(protocol, transport) -> None:
+    compress_obj = ZLibBackend.compressobj(level=ZLibBackend.Z_BEST_SPEED, wbits=-15)
     writer = WebSocketWriter(protocol, transport, compress=15)
-    await writer.send(b"text")
-    writer.transport.write.assert_called_with(b"\xc1\x06*I\xad(\x01\x00")
-    await writer.send(b"text")
-    writer.transport.write.assert_called_with(b"\xc1\x05*\x01b\x00\x00")
+
+    msg = (
+        compress_obj.compress(b"text") + compress_obj.flush(ZLibBackend.Z_SYNC_FLUSH)
+    ).removesuffix(b"\x00\x00\xff\xff")
+    await writer.send_frame(b"text", WSMsgType.TEXT)
+    writer.transport.write.assert_called_with(  # type: ignore[attr-defined]
+        b"\xc1" + len(msg).to_bytes(1, "big") + msg
+    )
+
+    msg = (
+        compress_obj.compress(b"text") + compress_obj.flush(ZLibBackend.Z_SYNC_FLUSH)
+    ).removesuffix(b"\x00\x00\xff\xff")
+    await writer.send_frame(b"text", WSMsgType.TEXT)
+    writer.transport.write.assert_called_with(  # type: ignore[attr-defined]
+        b"\xc1" + len(msg).to_bytes(1, "big") + msg
+    )
 
 
+@pytest.mark.usefixtures("parametrize_zlib_backend")
 async def test_send_compress_text_notakeover(protocol, transport) -> None:
+    compress_obj = ZLibBackend.compressobj(level=ZLibBackend.Z_BEST_SPEED, wbits=-15)
     writer = WebSocketWriter(protocol, transport, compress=15, notakeover=True)
-    await writer.send(b"text")
-    writer.transport.write.assert_called_with(b"\xc1\x06*I\xad(\x01\x00")
-    await writer.send(b"text")
-    writer.transport.write.assert_called_with(b"\xc1\x06*I\xad(\x01\x00")
+
+    msg = (
+        compress_obj.compress(b"text") + compress_obj.flush(ZLibBackend.Z_FULL_FLUSH)
+    ).removesuffix(b"\x00\x00\xff\xff")
+    await writer.send_frame(b"text", WSMsgType.TEXT)
+    writer.transport.write.assert_called_with(  # type: ignore[attr-defined]
+        b"\xc1" + len(msg).to_bytes(1, "big") + msg
+    )
+    await writer.send_frame(b"text", WSMsgType.TEXT)
+    writer.transport.write.assert_called_with(  # type: ignore[attr-defined]
+        b"\xc1" + len(msg).to_bytes(1, "big") + msg
+    )
 
 
 async def test_send_compress_text_per_message(protocol, transport) -> None:
     writer = WebSocketWriter(protocol, transport)
-    await writer.send(b"text", compress=15)
-    writer.transport.write.assert_called_with(b"\xc1\x06*I\xad(\x01\x00")
-    await writer.send(b"text")
-    writer.transport.write.assert_called_with(b"\x81\x04text")
-    await writer.send(b"text", compress=15)
-    writer.transport.write.assert_called_with(b"\xc1\x06*I\xad(\x01\x00")
+    await writer.send_frame(b"text", WSMsgType.TEXT, compress=15)
+    writer.transport.write.assert_called_with(b"\xc1\x06*I\xad(\x01\x00")  # type: ignore[attr-defined]
+    await writer.send_frame(b"text", WSMsgType.TEXT)
+    writer.transport.write.assert_called_with(b"\x81\x04text")  # type: ignore[attr-defined]
+    await writer.send_frame(b"text", WSMsgType.TEXT, compress=15)
+    writer.transport.write.assert_called_with(b"\xc1\x06*I\xad(\x01\x00")  # type: ignore[attr-defined]
 
 
 @pytest.mark.parametrize(
@@ -117,6 +142,7 @@ async def test_send_compress_text_per_message(protocol, transport) -> None:
         (32, lambda count: 64 + count if count % 2 else count),
     ),
 )
+@pytest.mark.usefixtures("parametrize_zlib_backend")
 async def test_concurrent_messages(
     protocol: Any,
     transport: Any,
@@ -141,10 +167,11 @@ async def test_concurrent_messages(
       and in the executor
     """
     with mock.patch(
-        "aiohttp.http_websocket.WEBSOCKET_MAX_SYNC_CHUNK_SIZE", max_sync_chunk_size
+        "aiohttp._websocket.writer.WEBSOCKET_MAX_SYNC_CHUNK_SIZE", max_sync_chunk_size
     ):
         writer = WebSocketWriter(protocol, transport, compress=15)
-        queue: DataQueue[WSMessage] = DataQueue(asyncio.get_running_loop())
+        loop = asyncio.get_running_loop()
+        queue = WebSocketDataQueue(mock.Mock(_reading_paused=False), 2**16, loop=loop)
         reader = WebSocketReader(queue, 50000)
         writers = []
         payloads = []
@@ -152,7 +179,7 @@ async def test_concurrent_messages(
             point = payload_point_generator(count)
             payload = bytes((point,)) * point
             payloads.append(payload)
-            writers.append(writer.send(payload, binary=True))
+            writers.append(writer.send_frame(payload, WSMsgType.BINARY))
         await asyncio.gather(*writers)
 
     for call in writer.transport.write.call_args_list:
