@@ -3,36 +3,35 @@ import asyncio
 import os
 import socket
 import ssl
+from typing import TYPE_CHECKING, Dict, Optional
 from unittest import mock
 
 import pytest
+from _pytest.fixtures import SubRequest
 
 from aiohttp import web
 
-base_worker = pytest.importorskip("aiohttp.worker")
+if TYPE_CHECKING:
+    from aiohttp import worker as base_worker
+else:
+    base_worker = pytest.importorskip("aiohttp.worker")
 
 
 try:
     import uvloop
 except ImportError:
-    uvloop = None
+    uvloop = None  # type: ignore[assignment]
 
 
 WRONG_LOG_FORMAT = '%a "%{Referrer}i" %(h)s %(l)s %s'
 ACCEPTABLE_LOG_FORMAT = '%a "%{Referrer}i" %s'
 
 
-# tokio event loop does not allow to override attributes
-def skip_if_no_dict(loop):
-    if not hasattr(loop, "__dict__"):
-        pytest.skip("can not override loop attributes")
-
-
 class BaseTestWorker:
-    def __init__(self):
-        self.servers = {}
+    def __init__(self) -> None:
+        self.servers: Dict[object, object] = {}
         self.exit_code = 0
-        self._notify_waiter = None
+        self._notify_waiter: Optional[asyncio.Future[bool]] = None
         self.cfg = mock.Mock()
         self.cfg.graceful_timeout = 100
         self.pid = "pid"
@@ -55,14 +54,16 @@ if uvloop is not None:
 
 
 @pytest.fixture(params=PARAMS)
-def worker(request, loop):
+def worker(
+    request: SubRequest, loop: asyncio.AbstractEventLoop
+) -> base_worker.GunicornWebWorker:
     asyncio.set_event_loop(loop)
     ret = request.param()
     ret.notify = mock.Mock()
-    return ret
+    return ret  # type: ignore[no-any-return]
 
 
-def test_init_process(worker) -> None:
+def test_init_process(worker: base_worker.GunicornWebWorker) -> None:
     with mock.patch("aiohttp.worker.asyncio") as m_asyncio:
         try:
             worker.init_process()
@@ -74,11 +75,14 @@ def test_init_process(worker) -> None:
         assert m_asyncio.set_event_loop.called
 
 
-def test_run(worker, loop) -> None:
+def test_run(
+    worker: base_worker.GunicornWebWorker, loop: asyncio.AbstractEventLoop
+) -> None:
     worker.log = mock.Mock()
     worker.cfg = mock.Mock()
     worker.cfg.access_log_format = ACCEPTABLE_LOG_FORMAT
     worker.cfg.is_ssl = False
+    worker.cfg.graceful_timeout = 100
     worker.sockets = []
 
     worker.loop = loop
@@ -88,16 +92,19 @@ def test_run(worker, loop) -> None:
     assert loop.is_closed()
 
 
-def test_run_async_factory(worker, loop) -> None:
+def test_run_async_factory(
+    worker: base_worker.GunicornWebWorker, loop: asyncio.AbstractEventLoop
+) -> None:
     worker.log = mock.Mock()
     worker.cfg = mock.Mock()
     worker.cfg.access_log_format = ACCEPTABLE_LOG_FORMAT
     worker.cfg.is_ssl = False
+    worker.cfg.graceful_timeout = 100
     worker.sockets = []
     app = worker.wsgi
 
-    async def make_app():
-        return app
+    async def make_app() -> web.Application:
+        return app  # type: ignore[no-any-return]
 
     worker.wsgi = make_app
 
@@ -109,7 +116,9 @@ def test_run_async_factory(worker, loop) -> None:
     assert loop.is_closed()
 
 
-def test_run_not_app(worker, loop) -> None:
+def test_run_not_app(
+    worker: base_worker.GunicornWebWorker, loop: asyncio.AbstractEventLoop
+) -> None:
     worker.log = mock.Mock()
     worker.cfg = mock.Mock()
     worker.cfg.access_log_format = ACCEPTABLE_LOG_FORMAT
@@ -123,32 +132,24 @@ def test_run_not_app(worker, loop) -> None:
     assert loop.is_closed()
 
 
-def test_handle_quit(worker, loop) -> None:
-    worker.loop = mock.Mock()
-    worker.handle_quit(object(), object())
-    assert not worker.alive
-    assert worker.exit_code == 0
-    worker.loop.call_later.asset_called_with(0.1, worker._notify_waiter_done)
-
-
-def test_handle_abort(worker) -> None:
+def test_handle_abort(worker: base_worker.GunicornWebWorker) -> None:
     with mock.patch("aiohttp.worker.sys") as m_sys:
-        worker.handle_abort(object(), object())
+        worker.handle_abort(0, None)
         assert not worker.alive
         assert worker.exit_code == 1
         m_sys.exit.assert_called_with(1)
 
 
-def test__wait_next_notify(worker) -> None:
-    worker.loop = mock.Mock()
-    worker._notify_waiter_done = mock.Mock()
-    fut = worker._wait_next_notify()
+def test__wait_next_notify(worker: base_worker.GunicornWebWorker) -> None:
+    worker.loop = mloop = mock.create_autospec(asyncio.AbstractEventLoop)
+    with mock.patch.object(worker, "_notify_waiter_done", autospec=True):
+        fut = worker._wait_next_notify()
 
-    assert worker._notify_waiter == fut
-    worker.loop.call_later.assert_called_with(1.0, worker._notify_waiter_done, fut)
+        assert worker._notify_waiter == fut
+        mloop.call_later.assert_called_with(1.0, worker._notify_waiter_done, fut)
 
 
-def test__notify_waiter_done(worker) -> None:
+def test__notify_waiter_done(worker: base_worker.GunicornWebWorker) -> None:
     worker._notify_waiter = None
     worker._notify_waiter_done()
     assert worker._notify_waiter is None
@@ -161,7 +162,9 @@ def test__notify_waiter_done(worker) -> None:
     waiter.set_result.assert_called_with(True)
 
 
-def test__notify_waiter_done_explicit_waiter(worker) -> None:
+def test__notify_waiter_done_explicit_waiter(
+    worker: base_worker.GunicornWebWorker,
+) -> None:
     worker._notify_waiter = None
     assert worker._notify_waiter is None
 
@@ -175,7 +178,7 @@ def test__notify_waiter_done_explicit_waiter(worker) -> None:
     assert not waiter2.set_result.called
 
 
-def test_init_signals(worker) -> None:
+def test_init_signals(worker: base_worker.GunicornWebWorker) -> None:
     worker.loop = mock.Mock()
     worker.init_signals()
     assert worker.loop.add_signal_handler.called
@@ -191,29 +194,31 @@ def test_init_signals(worker) -> None:
         ),
     ],
 )
-def test__get_valid_log_format_ok(worker, source, result) -> None:
+def test__get_valid_log_format_ok(
+    worker: base_worker.GunicornWebWorker, source: str, result: str
+) -> None:
     assert result == worker._get_valid_log_format(source)
 
 
-def test__get_valid_log_format_exc(worker) -> None:
+def test__get_valid_log_format_exc(worker: base_worker.GunicornWebWorker) -> None:
     with pytest.raises(ValueError) as exc:
         worker._get_valid_log_format(WRONG_LOG_FORMAT)
     assert "%(name)s" in str(exc.value)
 
 
-async def test__run_ok_parent_changed(worker, loop, aiohttp_unused_port) -> None:
-    skip_if_no_dict(loop)
-
+async def test__run_ok_parent_changed(
+    worker: base_worker.GunicornWebWorker,
+    loop: asyncio.AbstractEventLoop,
+    unused_port_socket: socket.socket,
+) -> None:
     worker.ppid = 0
     worker.alive = True
-    sock = socket.socket()
-    addr = ("localhost", aiohttp_unused_port())
-    sock.bind(addr)
+    sock = unused_port_socket
     worker.sockets = [sock]
     worker.log = mock.Mock()
     worker.loop = loop
+    worker.max_requests = 0
     worker.cfg.access_log_format = ACCEPTABLE_LOG_FORMAT
-    worker.cfg.max_requests = 0
     worker.cfg.is_ssl = False
 
     await worker._run()
@@ -222,24 +227,25 @@ async def test__run_ok_parent_changed(worker, loop, aiohttp_unused_port) -> None
     worker.log.info.assert_called_with("Parent changed, shutting down: %s", worker)
 
 
-async def test__run_exc(worker, loop, aiohttp_unused_port) -> None:
-    skip_if_no_dict(loop)
-
+async def test__run_exc(
+    worker: base_worker.GunicornWebWorker,
+    loop: asyncio.AbstractEventLoop,
+    unused_port_socket: socket.socket,
+) -> None:
     worker.ppid = os.getppid()
     worker.alive = True
-    sock = socket.socket()
-    addr = ("localhost", aiohttp_unused_port())
-    sock.bind(addr)
+    sock = unused_port_socket
     worker.sockets = [sock]
     worker.log = mock.Mock()
     worker.loop = loop
+    worker.max_requests = 0
     worker.cfg.access_log_format = ACCEPTABLE_LOG_FORMAT
-    worker.cfg.max_requests = 0
     worker.cfg.is_ssl = False
 
-    def raiser():
+    def raiser() -> None:
         waiter = worker._notify_waiter
         worker.alive = False
+        assert waiter is not None
         waiter.set_exception(RuntimeError())
 
     loop.call_later(0.1, raiser)
@@ -249,8 +255,8 @@ async def test__run_exc(worker, loop, aiohttp_unused_port) -> None:
 
 
 def test__create_ssl_context_without_certs_and_ciphers(
-    worker,
-    tls_certificate_pem_path,
+    worker: base_worker.GunicornWebWorker,
+    tls_certificate_pem_path: str,
 ) -> None:
     worker.cfg.ssl_version = ssl.PROTOCOL_TLS_CLIENT
     worker.cfg.cert_reqs = ssl.CERT_OPTIONAL
@@ -263,8 +269,8 @@ def test__create_ssl_context_without_certs_and_ciphers(
 
 
 def test__create_ssl_context_with_ciphers(
-    worker,
-    tls_certificate_pem_path,
+    worker: base_worker.GunicornWebWorker,
+    tls_certificate_pem_path: str,
 ) -> None:
     worker.cfg.ssl_version = ssl.PROTOCOL_TLS_CLIENT
     worker.cfg.cert_reqs = ssl.CERT_OPTIONAL
@@ -277,9 +283,9 @@ def test__create_ssl_context_with_ciphers(
 
 
 def test__create_ssl_context_with_ca_certs(
-    worker,
-    tls_ca_certificate_pem_path,
-    tls_certificate_pem_path,
+    worker: base_worker.GunicornWebWorker,
+    tls_ca_certificate_pem_path: str,
+    tls_certificate_pem_path: str,
 ) -> None:
     worker.cfg.ssl_version = ssl.PROTOCOL_TLS_CLIENT
     worker.cfg.cert_reqs = ssl.CERT_OPTIONAL
